import logging

from skew.resources.aws import AWSResource


LOG = logging.getLogger(__name__)


class CloudfrontResource(AWSResource):

    @property
    def arn(self):
        return 'arn:aws:%s::%s:%s/%s' % (
            self._client.service_name,
            self._client.account_id, self.resourcetype, self.id)


class Distribution(CloudfrontResource):

    class Meta(object):
        service = 'cloudfront'
        type = 'distribution'
        enum_spec = ('list_distributions', 'DistributionList.Items[]', None)
        detail_spec = None
        id = 'Id'
        tags_spec = ('list_tags_for_resource', 'Tags.Items[]',
                     'Resource', 'arn')
        name = 'DomainName'
        filter_name = None
        date = 'LastModifiedTime'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['Id']



# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource


class Application(AWSResource):

    class Meta(object):
        service = 'elasticbeanstalk'
        type = 'application'
        enum_spec = ('describe_applications', 'Applications', None)
        detail_spec = None
        id = 'ApplicationName'
        filter_name = None
        filter_type = None
        name = 'ApplicationName'
        date = None
        dimension = None

class Environment(AWSResource):

    class Meta(object):
        service = 'elasticbeanstalk'
        type = 'environment'
        enum_spec = ('describe_environments', 'Environments', None)
        detail_spec = None
        id = 'EnvironmentName'
        filter_name = None
        filter_type = None
        name = 'EnvironmentName'
        date = None
        dimension = None


# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource

import jmespath

class DeliveryStream(AWSResource):

    class Meta(object):
        service = 'firehose'
        type = 'deliverystream'
        enum_spec = ('list_delivery_streams', 'DeliveryStreamNames', None)
        detail_spec = ('describe_delivery_stream', 'DeliveryStreamName', 'DeliveryStreamDescription')
        id = 'DeliveryStreamName'
        filter_name = None
        filter_type = None
        name = 'DeliveryStreamName'
        date = 'CreateTimestamp'
        dimension = 'DeliveryStreamName'

    def __init__(self, client, data, query=None):
        super(DeliveryStream, self).__init__(client, data, query)
        self._id = data
        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: self.id}
        data = client.call(detail_op, **params)
        self.data = jmespath.search(detail_path, data)



# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging

import jmespath

from skew.resources.aws import AWSResource

LOG = logging.getLogger(__name__)


class Topic(AWSResource):

    class Meta(object):
        service = 'sns'
        type = 'topic'
        enum_spec = ('list_topics', 'Topics', None)
        detail_spec = ('get_topic_attributes', 'TopicArn', 'Attributes')
        id = 'TopicArn'
        filter_name = None
        filter_type = None
        name = 'DisplayName'
        date = None
        dimension = 'TopicName'

    @classmethod
    def filter(cls, arn, resource_id, data):
        topic_arn = data.get(cls.Meta.id)
        LOG.debug('%s == %s', arn, topic_arn)
        return arn == topic_arn

    @property
    def arn(self):
        return self.data.get('TopicArn')

    def __init__(self, client, data, query=None):
        super(Topic, self).__init__(client, data, query)

        self._id = data['TopicArn'].split(':', 5)[5]

        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: data['TopicArn']}
        data = client.call(detail_op, **params)

        self.data = jmespath.search(detail_path, data)


class Subscription(AWSResource):

    invalid_arns = ['PendingConfirmation', 'Deleted']

    class Meta(object):
        service = 'sns'
        type = 'subscription'
        enum_spec = ('list_subscriptions', 'Subscriptions', None)
        detail_spec = ('get_subscription_attributes', 'SubscriptionArn',
                       'Attributes')
        id = 'SubscriptionArn'
        filter_name = None
        filter_type = None
        name = 'SubscriptionArn'
        date = None
        dimension = None

    @property
    def arn(self):
        return self.data.get('SubscriptionArn')

    @classmethod
    def enumerate(cls, arn, region, account, resource_id=None, **kwargs):
        resources = super(Subscription, cls).enumerate(
            arn, region, account, resource_id, **kwargs)

        return [r for r in resources if r.id not in cls.invalid_arns]

    def __init__(self, client, data, query=None):
        super(Subscription, self).__init__(client, data, query)

        if data['SubscriptionArn'] in self.invalid_arns:
            self._id = 'PendingConfirmation'
            return

        self._id = data['SubscriptionArn'].split(':', 6)[6]
        self._name = ""

        detail_op, param_name, detail_path = self.Meta.detail_spec
        params = {param_name: data['SubscriptionArn']}
        data = client.call(detail_op, **params)

        self.data = jmespath.search(detail_path, data)



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import jmespath
import logging
from skew.resources.aws import AWSResource

LOG = logging.getLogger(__name__)


class Alarm(AWSResource):

    class Meta(object):
        service = 'cloudwatch'
        type = 'alarm'
        enum_spec = ('describe_alarms', 'MetricAlarms', None)
        id = 'AlarmName'
        filter_name = 'AlarmNames'
        filter_type = 'list'
        detail_spec = None
        name = 'AlarmName'
        date = 'AlarmConfigurationUpdatedTimestamp'
        dimension = None

    @property
    def arn(self):
        return 'arn:aws:%s:%s:%s:%s:%s' % (
            self._client.service_name,
            self._client.region_name,
            self._client.account_id,
            self.resourcetype, self.id)


class LogGroup(AWSResource):

    class Meta(object):
        service = 'logs'
        type = 'log-group'
        enum_spec = ('describe_log_groups', 'logGroups[]', None)
        attr_spec = [
            ('describe_log_streams', 'logGroupName',
             'logStreams', 'logStreams'),
            ('describe_metric_filters', 'logGroupName',
             'metricFilters', 'metricFilters'),
            ('describe_subscription_filters', 'logGroupName',
             'subscriptionFilters', 'subscriptionFilters'),
            ('describe_queries', 'logGroupName',
             'queries', 'queries'),
        ]
        detail_spec = None
        id = 'logGroupName'
        tags_spec = ('list_tags_log_group', 'tags',
                     'logGroupName', 'id')
        filter_name = 'logGroupNamePrefix'
        filter_type = 'dict'
        name = 'logGroupName'
        date = 'creationTime'
        dimension = 'logGroupName'

    def __init__(self, client, data, query=None):
        super(LogGroup, self).__init__(client, data, query)
        self._data = data
        self._keys = []
        self._id = data['logGroupName']

        # add addition attribute data
        for attr in self.Meta.attr_spec:
            LOG.debug(attr)
            detail_op, param_name, detail_path, detail_key = attr
            params = {param_name: self._id}
            data = self._client.call(detail_op, **params)
            if not (detail_path is None):
                data = jmespath.search(detail_path, data)
            if 'ResponseMetadata' in data:
                del data['ResponseMetadata']
            self.data[detail_key] = data
            LOG.debug(data)

    @property
    def logGroupName(self):
        return self.data.get('logGroupName')



# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging

from skew.resources.aws import AWSResource

LOG = logging.getLogger(__name__)


class RestAPI(AWSResource):

    class Meta(object):
        service = 'apigateway'
        type = 'restapis'
        enum_spec = ('get_rest_apis', 'items', None)
        id = 'id'
        filter_name = None
        filter_type = None
        detail_spec = None
        name = 'name'
        date = 'createdDate'
        dimension = 'GatewayName'

    @classmethod
    def filter(cls, arn, resource_id, data):
        api_id = data.get(cls.Meta.id)
        LOG.debug('%s == %s', resource_id, api_id)
        return resource_id == api_id



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import jmespath
import logging
from skew.resources.aws import AWSResource

LOG = logging.getLogger(__name__)


class LoadBalancer(AWSResource):

    class Meta(object):
        service = 'elb'
        type = 'loadbalancer'
        enum_spec = ('describe_load_balancers',
                     'LoadBalancerDescriptions', None)
        detail_spec = None
        attr_spec = [
            ('describe_load_balancer_attributes', 'LoadBalancerName',
                'LoadBalancerAttributes', 'LoadBalancerAttributes'),
            ('describe_load_balancer_policies', 'LoadBalancerName',
                'PolicyDescriptions', 'PolicyDescriptions'),
        ]
        id = 'LoadBalancerName'
        filter_name = 'LoadBalancerNames'
        filter_type = 'list'
        name = 'DNSName'
        date = 'CreatedTime'
        dimension = 'LoadBalancerName'
        tags_spec = ('describe_tags', 'TagDescriptions[].Tags[]',
                     'LoadBalancerNames', 'id')

    def __init__(self, client, data, query=None):
        super(LoadBalancer, self).__init__(client, data, query)
        self._id = data['LoadBalancerName']

        # add addition attribute data
        for attr in self.Meta.attr_spec:
            LOG.debug(attr)
            detail_op, param_name, detail_path, detail_key = attr
            params = {param_name: self._id}
            data = self._client.call(detail_op, **params)
            if not (detail_path is None):
                data = jmespath.search(detail_path, data)
            if 'ResponseMetadata' in data:
                del data['ResponseMetadata']
            self.data[detail_key] = data
            LOG.debug(data)



#!/usr/bin/env python

from setuptools import setup, find_packages

import os

requires = [
    'boto3>=1.2.3',
    'six>=1.9.0',
    'python-dateutil>=2.1,<3.0.0',
    'PyYAML>=3.11']


here = os.path.dirname(os.path.realpath(__file__))
with open(os.path.join(here, 'README.md'), encoding='utf-8') as readme:
    long_description = readme.read()


setup(
    name='skew',
    version=open(os.path.join('skew', '_version')).read().strip(),
    description='A flat address space for all your AWS resources.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Mitch Garnaat',
    author_email='mitch@garnaat.com',
    url='https://github.com/scopely-devops/skew',
    packages=find_packages(exclude=['tests*']),
    package_data={'skew': ['_version']},
    package_dir={'skew': 'skew'},
    install_requires=requires,
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3'
    ],
)



# Copyright (c) 2014 Scopely, Inc.
# Copyright (c) 2015 Mitch Garnaat
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging
import jmespath
from skew.resources.aws import AWSResource

LOG = logging.getLogger(__name__)


class IAMResource(AWSResource):

    @property
    def arn(self):
        return 'arn:aws:%s::%s:%s/%s' % (
            self._client.service_name,
            self._client.account_id, self.resourcetype, self.name)


class Group(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'group'
        enum_spec = ('list_groups', 'Groups', None)
        detail_spec = None
        id = 'GroupId'
        name = 'GroupName'
        filter_name = None
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['GroupName']


class User(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'user'
        enum_spec = ('list_users', 'Users', None)
        detail_spec = ('get_user', 'UserName', 'User')
        attr_spec = [
            ('list_access_keys', 'UserName',
                'AccessKeyMetadata', 'AccessKeyMetadata'),
            ('list_groups_for_user', 'UserName',
                'Groups', 'Groups'),
            ('list_user_policies', 'UserName',
                'PolicyNames', 'PolicyNames'),
            ('list_attached_user_policies', 'UserName',
                'AttachedPolicies', 'AttachedPolicies'),
            ('list_ssh_public_keys', 'UserName',
                'SSHPublicKeys', 'SSHPublicKeys'),
            # ('list_mfa_devices', 'UserName', 'MFADevices', 'MFADevices'),
        ]
        id = 'UserId'
        filter_name = None
        name = 'UserName'
        date = 'CreateDate'
        dimension = None
        tags_spec = ('list_user_tags', 'Tags[]',
                     'UserName', 'name')

    def __init__(self, client, data, query=None):
        super(User, self).__init__(client, data, query)

        LOG.debug(data)
        # add details
        if self.Meta.detail_spec is not None:
            detail_op, param_name, detail_path = self.Meta.detail_spec
            params = {param_name: self.data[param_name]}
            data = client.call(detail_op, **params)
            self.data = jmespath.search(detail_path, data)

        # add attribute data
        if self.Meta.attr_spec is not None:
            for attr in self.Meta.attr_spec:
                LOG.debug(attr)
                LOG.debug(data)
                detail_op, param_name, detail_path, detail_key = attr
                params = {param_name: self.data[param_name]}
                tmp_data = self._client.call(detail_op, **params)
                if not (detail_path is None):
                    tmp_data = jmespath.search(detail_path, tmp_data)
                if 'ResponseMetadata' in tmp_data:
                    del tmp_data['ResponseMetadata']
                self.data[detail_key] = tmp_data
                LOG.debug(data)

            # retrieve all of the inline IAM policies
            if 'PolicyNames' in self.data \
                and self.data['PolicyNames']:
                tmp_dict = {}
                for policy_name in self.data['PolicyNames']:
                    params = {
                        'UserName': self.data['UserName'],
                        'PolicyName': policy_name
                    }
                    tmp_data = self._client.call('get_user_policy', **params)
                    tmp_data = jmespath.search('PolicyDocument', tmp_data)
                    tmp_dict[policy_name] = tmp_data
                self.data['PolicyNames'] = tmp_dict

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['UserName']


class Role(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'role'
        enum_spec = ('list_roles', 'Roles', None)
        detail_spec = None
        id = 'RoleId'
        filter_name = None
        name = 'RoleName'
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['RoleName']


class InstanceProfile(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'instance-profile'
        enum_spec = ('list_instance_profiles', 'InstanceProfiles', None)
        detail_spec = None
        id = 'InstanceProfileId'
        filter_name = None
        name = 'InstanceProfileId'
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['InstanceProfileId']


class Policy(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'policy'
        enum_spec = ('list_policies', 'Policies', None)
        detail_spec = None
        id = 'PolicyArn'
        filter_name = None
        name = 'PolicyName'
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['PolicyName']


class ServerCertificate(IAMResource):

    class Meta(object):
        service = 'iam'
        type = 'server-certificate'
        enum_spec = ('list_server_certificates',
                     'ServerCertificateMetadataList',
                     None)
        detail_spec = None
        id = 'ServerCertificateId'
        filter_name = None
        name = 'ServerCertificateName'
        date = 'Expiration'
        dimension = None

    @classmethod
    def filter(cls, arn, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['ServerCertificateName']
